@@ -69,6 +69,25 @@ static void parseAddrList(const QVarLengthArray<QByteArray, 16> &addrList, KMime
     hdr->setMailboxes(mboxList);
 }
 
+static void parseAddrList(const QVarLengthArray<QByteArray, 16> &addrList, KMime::Headers::Generics::SingleMailbox *hdr, int version, StringPool &pool)
+{
+    const int count = addrList.count();
+    if (count != 1) {
+        return;
+    }
+    KMime::Types::Mailbox::List mboxList;
+    mboxList.reserve(count);
+
+    for (int i = 0; i < count; ++i) {
+        auto addrField = parseMailbox(addrList[i], version, pool);
+        if (addrField) {
+            mboxList.push_back(std::move(*addrField));
+        }
+    }
+
+    hdr->setMailbox(mboxList[0]);
+}
+
 static void parseAddrList(const QVarLengthArray<QByteArray, 16> &addrList, KMime::Headers::Generics::AddressList *hdr, int version, StringPool &pool)
 {
     const int count = addrList.count();
@@ -90,7 +109,7 @@ static void parseAddrList(const QVarLengthArray<QByteArray, 16> &addrList, KMime
 // v2 format
 [[nodiscard]] static KMime::Types::Mailbox parseMailbox(QDataStream &stream, StringPool &pool)
 {
-    QString str;
+    static thread_local QString str;
     KMime::Types::Mailbox mbox;
     KMime::Types::AddrSpec addrSpec;
     stream >> str;
@@ -103,12 +122,16 @@ static void parseAddrList(const QVarLengthArray<QByteArray, 16> &addrList, KMime
     return mbox;
 }
 
-static void parseAddrList(QDataStream &stream, KMime::Headers::Generics::MailboxList *hdr, int version, StringPool &pool)
+template<typename T>
+static void parseMailboxList(QDataStream &stream, T hdrFunc, KMime::Message::Ptr &msg, int version, StringPool &pool)
 {
     Q_UNUSED(version)
 
     int count = 0;
     stream >> count;
+    if (count == 0) {
+        return;
+    }
     KMime::Types::Mailbox::List mboxList;
     mboxList.reserve(count);
 
@@ -116,15 +139,38 @@ static void parseAddrList(QDataStream &stream, KMime::Headers::Generics::Mailbox
         mboxList.push_back(parseMailbox(stream, pool));
     }
 
-    hdr->setMailboxes(mboxList);
+    (msg.get()->*(hdrFunc))(true)->setMailboxes(mboxList);
 }
 
-static void parseAddrList(QDataStream &stream, KMime::Headers::Generics::AddressList *hdr, int version, StringPool &pool)
+template<typename T>
+static void parseSingleMailbox(QDataStream &stream, T hdrFunc, KMime::Message::Ptr &msg, int version, StringPool &pool)
 {
     Q_UNUSED(version)
 
     int count = 0;
     stream >> count;
+    if (count == 0) {
+        return;
+    }
+
+    // we need to consume the entire stream, even if it contains more than one element...
+    for (int i = 0; i < count; ++i) {
+        if (!i) {
+            (msg.get()->*(hdrFunc))(true)->setMailbox(parseMailbox(stream, pool));
+        }
+    }
+}
+
+template<typename T>
+static void parseAddrList(QDataStream &stream, T hdrFunc, KMime::Message::Ptr &msg, int version, StringPool &pool)
+{
+    Q_UNUSED(version)
+
+    int count = 0;
+    stream >> count;
+    if (count == 0) {
+        return;
+    }
     KMime::Types::AddressList addrList;
     addrList.reserve(count);
 
@@ -134,7 +180,7 @@ static void parseAddrList(QDataStream &stream, KMime::Headers::Generics::Address
         addrList.push_back(std::move(addr));
     }
 
-    hdr->setAddressList(addrList);
+    (msg.get()->*(hdrFunc))(true)->setAddressList(addrList);
 }
 
 bool SerializerPluginMail::deserialize(Item &item, const QByteArray &label, QIODevice &data, int version)
@@ -217,7 +263,9 @@ bool SerializerPluginMail::deserialize(Item &item, const QByteArray &label, QIOD
                 parseAddrList(addrList, msg->bcc(), version, m_stringPool);
             }
             // in-reply-to
-            msg->inReplyTo()->from7BitString(env[8]);
+            if (!env[8].isEmpty()) {
+                msg->inReplyTo()->from7BitString(env[8]);
+            }
             // message id
             msg->messageID()->from7BitString(env[9]);
             // references
@@ -236,22 +284,26 @@ bool SerializerPluginMail::deserialize(Item &item, const QByteArray &label, QIOD
 
             QString inReplyTo;
             stream >> inReplyTo;
-            msg->inReplyTo()->fromUnicodeString(inReplyTo);
+            if (!inReplyTo.isEmpty()) {
+                msg->inReplyTo()->fromUnicodeString(inReplyTo);
+            }
             stream >> str;
             msg->messageID()->fromUnicodeString(str);
             stream >> str;
-            if (str == inReplyTo) {
-                msg->references()->fromIdent(msg->inReplyTo());
-            } else {
-                msg->references()->fromUnicodeString(str);
+            if (!str.isEmpty()) {
+                if (str == inReplyTo) {
+                    msg->references()->fromIdent(msg->inReplyTo());
+                } else {
+                    msg->references()->fromUnicodeString(str);
+                }
             }
 
-            parseAddrList(stream, msg->from(), version, m_stringPool);
-            parseAddrList(stream, msg->sender(), version, m_stringPool);
-            parseAddrList(stream, msg->replyTo(), version, m_stringPool);
-            parseAddrList(stream, msg->to(), version, m_stringPool);
-            parseAddrList(stream, msg->cc(), version, m_stringPool);
-            parseAddrList(stream, msg->bcc(), version, m_stringPool);
+            parseMailboxList(stream, qOverload<bool>(&KMime::Message::from), msg, version, m_stringPool);
+            parseSingleMailbox(stream, qOverload<bool>(&KMime::Message::sender), msg, version, m_stringPool);
+            parseAddrList(stream, qOverload<bool>(&KMime::Message::replyTo), msg, version, m_stringPool);
+            parseAddrList(stream, qOverload<bool>(&KMime::Message::to), msg, version, m_stringPool);
+            parseAddrList(stream, qOverload<bool>(&KMime::Message::cc), msg, version, m_stringPool);
+            parseAddrList(stream, qOverload<bool>(&KMime::Message::bcc), msg, version, m_stringPool);
 
             if (stream.status() == QDataStream::ReadCorruptData || stream.status() == QDataStream::ReadPastEnd) {
                 qCWarning(AKONADI_SERIALIZER_MAIL_LOG) << "Akonadi KMime Deserializer: Got invalid envelope";
@@ -266,6 +318,11 @@ bool SerializerPluginMail::deserialize(Item &item, const QByteArray &label, QIOD
 template<typename T>
 static void serializeAddrList(QDataStream &stream, T *hdr)
 {
+    if (!hdr) {
+        stream << (qint32)0;
+        return;
+    }
+
     const KMime::Types::Mailbox::List mb = hdr->mailboxes();
     stream << (qint32)mb.size();
     for (const KMime::Types::Mailbox &mbox : mb) {
@@ -273,6 +330,24 @@ static void serializeAddrList(QDataStream &stream, T *hdr)
     }
 }
 
+static void serializeAddr(QDataStream &stream, KMime::Headers::Generics::SingleMailbox *hdr)
+{
+    if (!hdr) {
+        stream << (qint32)0;
+        return;
+    }
+
+    stream << (qint32)1;
+    const auto mbox = hdr->mailbox();
+    stream << mbox.name() << mbox.addrSpec().localPart << mbox.addrSpec().domain;
+}
+
+template<typename T>
+static QString asUnicodeString(T *hdr)
+{
+    return hdr ? hdr->asUnicodeString() : QString();
+}
+
 void SerializerPluginMail::serialize(const Item &item, const QByteArray &label, QIODevice &data, int &version)
 {
     version = 1;
@@ -283,14 +358,14 @@ void SerializerPluginMail::serialize(const Item &item, const QByteArray &label,
     } else if (label == MessagePart::Envelope) {
         version = 2;
         QDataStream stream(&data);
-        stream << m->date()->dateTime() << m->subject()->asUnicodeString() << m->inReplyTo()->asUnicodeString() << m->messageID()->asUnicodeString()
-               << m->references()->asUnicodeString();
-        serializeAddrList(stream, m->from());
-        serializeAddrList(stream, m->sender());
-        serializeAddrList(stream, m->replyTo());
-        serializeAddrList(stream, m->to());
-        serializeAddrList(stream, m->cc());
-        serializeAddrList(stream, m->bcc());
+        stream << m->date()->dateTime() << asUnicodeString(m->subject(false)) << asUnicodeString(m->inReplyTo(false)) << asUnicodeString(m->messageID(false))
+               << asUnicodeString(m->references(false));
+        serializeAddrList(stream, m->from(false));
+        serializeAddr(stream, m->sender(false));
+        serializeAddrList(stream, m->replyTo(false));
+        serializeAddrList(stream, m->to(false));
+        serializeAddrList(stream, m->cc(false));
+        serializeAddrList(stream, m->bcc(false));
     } else if (label == MessagePart::Header) {
         data.write(m->head());
     }
